#include "sqlite3helper.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

namespace robot {
namespace common {

Sqlite3helper* Sqlite3helper::_pInstance = 0;


Sqlite3helper& Sqlite3helper::Instance(void)
{
  if (_pInstance != 0)
    return *_pInstance;
}

Sqlite3helper::Sqlite3helper()
{

}

Sqlite3helper::~Sqlite3helper()
{
  _pInstance = 0;
}

// 如果dbname为空或者连接数据库失败，返回NULL
sqlite3* Sqlite3helper::open(const char *dbname)
{
  if (strcmp("", dbname) == 0 || !dbname)
    return NULL;

  sqlite3 *pSqlite = NULL;
  if (sqlite3_open(dbname, &pSqlite) != SQLITE_OK){
    LOG(ERROR)<<"connect database error:"<<sqlite3_errmsg(pSqlite);
  }
  else {
    LOG(INFO)<< "connect database success.";
  }
  return pSqlite;
}

// 如果表不存在，返回0；存在返回1；查询失败返回-1
int Sqlite3helper::isTableExits(sqlite3 *p, const char *tableName)
{
  if (!p || strcmp("", tableName) == 0 || !tableName)
    return 0;

  int result;
  char **pResult;
  int nRow;
  int nCol;
  char *zErrorMsg = NULL;
  char sql[100] = "SELECT COUNT(*) FROM sqlite_master where type='table' and name='";
  strcat(sql,  tableName);
  strcat(sql, "'");

  int ret = sqlite3_get_table(p, sql, &pResult, &nRow, &nCol, &zErrorMsg);
  if (ret != SQLITE_OK) {
     LOG(ERROR)<<"check table errror , error=:"<<zErrorMsg<<"\n";
    result = -1;
  }
  else {
    int nIndex = nCol;
    for (int i = 0; i<nRow; i++) {
      for (int j = 0; j<nCol; j++) {
        char *count = pResult[nIndex];
        ++nIndex;
        if (strcmp(count, "1") == 0) {
          sqlite3_free(zErrorMsg);
          sqlite3_free_table(pResult);
          LOG(INFO)<<"table is exited\n";
          return 1;
        }
      } // end for
    } // end for
  } // end else

  sqlite3_free(zErrorMsg);
  sqlite3_free_table(pResult);

  return 0;
}

int Sqlite3helper::createTable(sqlite3 *p, const char *sql)
{
  if (!p || strcmp("", sql) == 0 || !sql)
    return 0;

  int result;
  char *zErrorMsg = NULL;
  if (sqlite3_exec(p, sql, 0, 0, &zErrorMsg) != SQLITE_OK) {
    LOG(ERROR)<<"create table error :"<< zErrorMsg;
    result = 0;
  }
  else
    result = 1;

  sqlite3_free(zErrorMsg);
  return result;
}

int Sqlite3helper::transaction(sqlite3 *p)
{
  if (!p)
    return 0;

  int result;
  char *zErrorMsg = NULL;
  int ret = sqlite3_exec(p, "begin transaction", 0, 0, &zErrorMsg); // 开始一个事务
  if (ret != SQLITE_OK) {
    LOG(ERROR)<<"begin transaction error:"<< zErrorMsg;
    result = 0;
  }
  else
    result = 1;

  sqlite3_free(zErrorMsg);
  return result;
}

int Sqlite3helper::commitTransaction(sqlite3 *p)
{
  if (!p)
    return 0;

  int result;
  char *zErrorMsg = NULL;
  int ret = sqlite3_exec(p, "commit transaction", 0, 0, &zErrorMsg); // 开始一个事务
  if (ret != SQLITE_OK) {
    LOG(ERROR)<<"commit transaction error :%s"<< zErrorMsg;
    result = 0;
  }
  else
    result = 1;

  sqlite3_free(zErrorMsg);
  return result;
}

int Sqlite3helper::rollbackTransaction(sqlite3 *p)
{
  if (!p)
    return 0;

  int result;
  char *zErrorMsg = NULL;
  int ret = sqlite3_exec(p, "rollback transaction", 0, 0, &zErrorMsg); // 开始一个事务
  if (ret != SQLITE_OK) {
    LOG(ERROR)<<"rollback transaction error:"<<zErrorMsg;
    result = 0;
  }
  else
    result = 1;

  sqlite3_free(zErrorMsg);
  return result;
}

void Sqlite3helper::close(sqlite3 *p)
{
  if (p) {
    sqlite3_close(p);
    p = NULL;
  }
}

int Sqlite3helper::update(sqlite3 *p, const char *sql)
{
  if (!p || strcmp("", sql) == 0 || !sql)
    return 0;

  int result;
  char *zErrorMsg = NULL;
  if (sqlite3_exec(p, sql, 0, 0, &zErrorMsg) != SQLITE_OK) {
    LOG(ERROR)<<"add or update table record:"<< zErrorMsg;
    result = 0;
  }
  else
    result = 1;

  sqlite3_free(zErrorMsg);
  return result;
}

query_result_t* Sqlite3helper::query(sqlite3 *p, const char *sql)
{
  if (!p || strcmp("", sql) == 0 || !sql)
    return 0;

  query_result_t *query_result = NULL;
  char *zErrorMsg = NULL;
  char **dbResult;
  int nRow = 0;
  int nColumn = 0;

  if (sqlite3_get_table(p, sql, &dbResult, &nRow, &nColumn, &zErrorMsg) != SQLITE_OK)
    LOG(ERROR)<<"select table error"<< zErrorMsg;
  else
    query_result = new query_result_t(nRow, nColumn, dbResult);

  sqlite3_free(zErrorMsg);
  return query_result;
}

}  // namespace common
}  // namespace robot
